@@ -1,36 +1,79 @@
 use numbers::*;
-use super::Zero;
 use std::mem;
 
+/// SIMD methods which have `f32` or `f64` specific implementation.
 pub trait Simd<T>: Sized
     where T: Sized + Sync + Send
 {
+    /// The type of real valued array which matches a SIMD register.
     type Array;
+    
+    /// SIMD register to array.
     fn to_array(self) -> Self::Array;
+    
+    /// The type of complex valued array which matches a SIMD register.
     type ComplexArray;
+    
+    /// Number of elements in a SIMD register.
     fn len() -> usize;
+    
+    /// Loads a SIMD register from an array. If the end of the array is approached
+    /// then the load code wraps around to the beginning and starts to load the first 
+    /// elements again.
     fn load_wrap_unchecked(array: &[T], idx: usize) -> Self;
+    
+    /// Creates a SIMD register loaded with a complex value.
     fn from_complex(value: Complex<T>) -> Self;
+    
+    /// Add a real number to the register.
     fn add_real(self, value: T) -> Self;
+    
+    /// Add a complex number to the register.
     fn add_complex(self, value: Complex<T>) -> Self;
+    
+    /// Scale the register by a real number.
     fn scale_real(self, value: T) -> Self;
+    
+    /// Scale the register by a complex number.
     fn scale_complex(self, value: Complex<T>) -> Self;
+    
+    /// Store the complex norm squared in the first half of the vector.
     fn complex_abs_squared(self) -> Self;
+    
+    /// Store the complex norm in the first half of the vector.
     fn complex_abs(self) -> Self;
+    
     /// Same as `complex_abs_squared` but stores the result
     /// as complex number
     fn complex_abs_squared2(self) -> Self;
+    
     /// Same as `complex_abs` but stores the result
-    /// as complex number
+    /// as complex number where the imaginary part is 0.
     fn complex_abs2(self) -> Self;
+    
+    /// Calculates the square root of the register.
     fn sqrt(self) -> Self;
+    
+    /// Stores the first half of the vector in an array.
+    /// Useful e.g. in combination with `complex_abs_squared`.
     fn store_half_unchecked(self, target: &mut [T], index: usize);
+    
+    /// Multiplies the register with a complex value.
     fn mul_complex(self, value: Self) -> Self;
+    
+    /// Divides the register by a complex value.
     fn div_complex(self, value: Self) -> Self;
+    
+    /// Calculates the sum of all register elements, assuming that they
+    /// are real valued.
     fn sum_real(&self) -> T;
+    
+    /// Calculates the sum of all register elements, assuming that they
+    /// are complex valued.
     fn sum_complex(&self) -> Complex<T>;
 }
 
+/// SIMD methods which share their implementation independent if it's a `f32` or `f64` register.
 pub trait SimdGeneric<T>: Simd<T>
     where T: Sized + Sync + Send
 {
@@ -40,44 +83,78 @@ pub trait SimdGeneric<T>: Simd<T>
     /// Center will contain most of the data.
     fn calc_data_alignment_reqs(array: &[T]) -> (usize, usize, usize);
 
+    /// Converts a real valued array which has exactly the size of a SIMD register
+    /// into a SIMD register.
     fn from_array(array: Self::Array) -> Self;
 
+    /// Converts the SIMD register into a complex valued array.
     fn to_complex_array(self) -> Self::ComplexArray;
-
+    
+    /// Converts a complex valued array which has exactly the size of a SIMD register
+    /// into a SIMD register.
     fn from_complex_array(array: Self::ComplexArray) -> Self;
 
+    /// Executed the given function on each element of the register.
+    /// Register elements are assumed to be real valued.
     fn iter_over_vector<F>(self, op: F) -> Self where F: FnMut(T) -> T;
 
+    /// Executed the given function on each element of the register.
+    /// Register elements are assumed to be complex valued.
     fn iter_over_complex_vector<F>(self, op: F) -> Self where F: FnMut(Complex<T>) -> Complex<T>;
 
+    /// Converts an array slice into a slice of SIMD registers.
     fn array_to_regs(array: &[T]) -> &[Self];
 
+    /// Converts a mutable array slice into a slice of mutable SIMD registers.
     fn array_to_regs_mut(array: &mut [T]) -> &mut [Self];
 
+    /// Loads a SIMD register from an array without any bound checks.
     fn load_unchecked(array: &[T], idx: usize) -> Self;
 
+    /// Stores a SIMD register into an array without any bound checks.
     fn store_unchecked(self, target: &mut [T], index: usize);
 
+    /// Returns one element from the register.
     fn extract(self, idx: u32) -> T;
 
+    /// Creates a new SIMD register where every element equals `value`.
     fn splat(value: T) -> Self;
 
+    /// Stores a SIMD register into an array.
     fn store(self, array: &mut [T], index: usize);
 }
 
-pub trait SimdApproximations<T> : Simd<T>
-    where T: Sized + Sync + Send {
+/// Approximated and faster implementation of some numeric standard function.
+/// The approximations are implemented based on SIMD registers.
+/// Refer to the documentation of the `ApproximatedOps` trait (which is part of 
+/// the public API of this lib) for some information about accuracy and speed.
+pub trait SimdApproximations<T>: Simd<T>
+    where T: Sized + Sync + Send
+{
+    /// Returns the natural logarithm of the number.
     fn ln_approx(self) -> Self;
 
+    /// Returns `e^(self)`, (the exponential function).
     fn exp_approx(self) -> Self;
 
+    /// Computes the sine of a number (in radians).
     fn sin_approx(self) -> Self;
 
+    /// Computes the cosine of a number (in radians).
     fn cos_approx(self) -> Self;
 
+    /// An implementation detail which leaked into the trait defintion
+    /// for convenience. Use `sin_approx` or `cos_approx` instead of this
+    /// function.
+    ///
+    /// Since the implementation of sine and cosine is almost identical 
+    /// the implementation is easier with a boolean `is_sin` flag which 
+    /// determines if the sine or cosine is requried.
     fn sin_cos_approx(self, is_sin: bool) -> Self;
 }
 
+/// Private struct copied over from the `simd` crate to implement the `load_unchecked` 
+/// and `store_unchecked` methods for the `SimdGeneric` trait.
 #[repr(packed)]
 #[derive(Debug, Copy, Clone)]
 struct Unalign<T>(T);
@@ -202,6 +279,7 @@ macro_rules! simd_generic_impl {
         }
     }
 }
+
 #[cfg(any(feature = "doc", feature="use_avx"))]
 mod avx;
 
@@ -214,7 +292,7 @@ mod sse;
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 pub use self::sse::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
 
-//#[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
+// #[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 mod approximations;
 
