@@ -1,10 +1,10 @@
 use std::mem;
 use std::ptr;
-use {RealNumber, array_to_complex_mut, memcpy, memzero};
+use {array_to_complex_mut, memcpy, memzero};
 use numbers::*;
 use multicore_support::*;
-use super::super::{VoidResult, Buffer, Owner, ErrorReason, NumberSpace,
-                   Domain, ResizeOps, DspVec, Vector, ToSliceMut, MetaData};
+use super::super::{VoidResult, Buffer, BufferBorrow, ErrorReason, NumberSpace, Domain, ResizeOps,
+                   DspVec, Vector, ToSliceMut, MetaData};
 
 /// This trait allows to reorganize the data by changing positions of the individual elements.
 pub trait ReorganizeDataOps<T>
@@ -38,28 +38,6 @@ pub trait ReorganizeDataOps<T>
     fn swap_halves(&mut self);
 }
 
-/// This trait allows to reorganize the data by changing positions of the individual elements.
-/// Deprecated since it requires a buffer which is never used since there would be no beneifit in using it.
-#[deprecated(since="0.4.1", note="please use `ReorganizeDataOps` instead which offers the same performance without a buffer")]
-pub trait ReorganizeDataOpsBuffered<S, T>
-    where T: RealNumber,
-          S: ToSliceMut<T>
-{
-    /// This function swaps both halves of the vector. This operation is also called FFT shift
-    /// Use it after a `plain_fft` to get a spectrum which is centered at `0 Hz`.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use basic_dsp_vector::*;
-    /// let mut vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0).to_real_time_vec();
-    /// let mut buffer = SingleBuffer::new();
-    /// vector.swap_halves_b(&mut buffer);
-    /// assert_eq!([5.0, 6.0, 7.0, 8.0, 1.0, 2.0, 3.0, 4.0], vector[..]);
-    /// ```
-    fn swap_halves_b<B>(&mut self, buffer: &mut B) where B: Buffer<S, T>;
-}
-
 /// An option which defines how a vector should be padded
 #[derive(Copy)]
 #[derive(Clone)]
@@ -133,8 +111,7 @@ pub trait InsertZerosOpsBuffered<S, T>
 {
     /// Appends zeros add the end of the vector until the vector has the size given in the
     /// points argument.
-    /// If `points` smaller than the `self.len()` then this operation won't do anything, however
-    /// in future it will raise an error
+    /// If `points` smaller than the `self.len()` then this operation will return an error.
     ///
     /// Note: Each point is two floating point numbers if the vector is complex.
     /// Note2: Adding zeros to the signal changes its power. If this function is used to
@@ -149,14 +126,18 @@ pub trait InsertZerosOpsBuffered<S, T>
     /// use basic_dsp_vector::*;
     /// let mut vector = vec!(1.0, 2.0).to_real_time_vec();
     /// let mut buffer = SingleBuffer::new();
-    /// vector.zero_pad_b(&mut buffer, 4, PaddingOption::End);
+    /// vector.zero_pad_b(&mut buffer, 4, PaddingOption::End).expect("Ignoring error handling in examples");
     /// assert_eq!([1.0, 2.0, 0.0, 0.0], vector[..]);
     /// let mut vector = vec!(1.0, 2.0).to_complex_time_vec();
-    /// vector.zero_pad_b(&mut buffer, 2, PaddingOption::End);
+    /// vector.zero_pad_b(&mut buffer, 2, PaddingOption::End).expect("Ignoring error handling in examples");
     /// assert_eq!([1.0, 2.0, 0.0, 0.0], vector[..]);
     /// ```
-    fn zero_pad_b<B>(&mut self, buffer: &mut B, points: usize, option: PaddingOption)
-        where B: Buffer<S, T>;
+    fn zero_pad_b<B>(&mut self,
+                     buffer: &mut B,
+                     points: usize,
+                     option: PaddingOption)
+                     -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>;
 
     /// Interleaves zeros `factor - 1`times after every vector element, so that the resulting
     /// vector will have a length of `self.len() * factor`.
@@ -177,7 +158,8 @@ pub trait InsertZerosOpsBuffered<S, T>
     /// vector.zero_interleave_b(&mut buffer, 2);
     /// assert_eq!([1.0, 2.0, 0.0, 0.0, 3.0, 4.0, 0.0, 0.0], vector[..]);
     /// ```
-    fn zero_interleave_b<B>(&mut self, buffer: &mut B, factor: u32) where B: Buffer<S, T>;
+    fn zero_interleave_b<B>(&mut self, buffer: &mut B, factor: u32)
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 /// Splits the data into several smaller pieces of equal size.
@@ -229,7 +211,8 @@ pub trait MergeOps {
 }
 
 fn reverse_array<T>(data: &mut [T])
-    where T: Copy {
+    where T: Copy
+{
     let len = data.len();
     // +1 makes sure that for odd numbers the first `lo` gets the extra item
     // (which is later on ignored)
@@ -240,7 +223,7 @@ fn reverse_array<T>(data: &mut [T])
 }
 
 impl<S, T, N, D> ReorganizeDataOps<T> for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
+    where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: Domain
@@ -262,20 +245,6 @@ impl<S, T, N, D> ReorganizeDataOps<T> for DspVec<S, T, N, D>
     }
 }
 
-#[allow(deprecated)]
-impl<S, T, N, D> ReorganizeDataOpsBuffered<S, T> for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
-          T: RealNumber,
-          N: NumberSpace,
-          D: Domain
-{
-    fn swap_halves_b<B>(&mut self, _: &mut B)
-        where B: Buffer<S, T>
-    {
-        self.swap_halves_priv(true);
-    }
-}
-
 macro_rules! zero_interleave {
     ($self_: ident, $buffer: ident, $step: ident, $tuple: expr) => {
         {
@@ -287,7 +256,7 @@ macro_rules! zero_interleave {
             let old_len = $self_.len();
             let new_len = step * old_len;
             $self_.valid_len = new_len;
-            let mut target = $buffer.get(new_len);
+            let mut target = $buffer.borrow(new_len);
 			{
 				let mut target = target.to_slice_mut();
 				let source = &$self_.data.to_slice();
@@ -317,8 +286,7 @@ macro_rules! zero_interleave {
             	});
 			}
 
-			mem::swap(&mut $self_.data, &mut target);
-			$buffer.free(target);
+			target.trade(&mut $self_.data);
         }
     }
 }
@@ -335,7 +303,7 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
         let step = if is_complex { 2 } else { 1 };
         let len = points * step;
         if len <= len_before {
-            return Ok(());
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
         try!(self.resize(len));
@@ -343,13 +311,13 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
         match option {
             PaddingOption::End => {
                 // Zero target
-                memzero(data, len_before .. len);
+                memzero(data, len_before..len);
                 Ok(())
             }
             PaddingOption::Surround => {
                 let diff = (len - len_before) / step;
                 let mut right = (diff - 1) / 2;
-                let mut left = diff - right ;
+                let mut left = diff - right;
                 if is_complex {
                     right *= 2;
                     left *= 2;
@@ -357,15 +325,15 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
 
                 memcpy(data, 0..len_before, left);
                 if right > 0 {
-                    memzero(data, len - right .. len);
+                    memzero(data, len - right..len);
                 }
-                memzero(data, 0 .. left);
+                memzero(data, 0..left);
                 Ok(())
             }
             PaddingOption::Center => {
                 let points_before = len_before / step;
                 let mut right = (points_before - 1) / 2;
-                let mut left = points_before - right ;
+                let mut left = points_before - right;
                 if is_complex {
                     right *= 2;
                     left *= 2;
@@ -375,7 +343,7 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
                     memcpy(data, left..left + right, len - right);
                 }
 
-                memzero(data, left .. len - right);
+                memzero(data, left..len - right);
                 Ok(())
             }
         }
@@ -421,22 +389,22 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
 
 
 impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
+    where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: Domain
 {
-    fn zero_pad_b<B>(&mut self, buffer: &mut B, points: usize, option: PaddingOption)
-        where B: Buffer<S, T>
+    fn zero_pad_b<B>(&mut self, buffer: &mut B, points: usize, option: PaddingOption) -> VoidResult
+        where B: for<'a> Buffer<'a, S, T>
     {
         let len_before = self.len();
         let is_complex = self.is_complex();
         let len = if is_complex { 2 * points } else { points };
         if len <= len_before {
-            return;
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
-        let mut target = buffer.get(len);
+        let mut target = buffer.borrow(len);
         {
             let data = self.data.to_slice();
             let target = target.to_slice_mut();
@@ -445,7 +413,7 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                 PaddingOption::End => {
                     // Zero target
                     &mut target[0..len_before].copy_from_slice(&data[0..len_before]);
-                    memzero(target, len_before .. len);
+                    memzero(target, len_before..len);
                 }
                 PaddingOption::Surround => {
                     let diff = (len - len_before) / if is_complex { 2 } else { 1 };
@@ -456,11 +424,11 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                         left *= 2;
                     }
 
-                    &mut target[left..left+len_before].copy_from_slice(&data[0..len_before]);
+                    &mut target[left..left + len_before].copy_from_slice(&data[0..len_before]);
                     if right > 0 {
-                        memzero(target, len - right .. len);
+                        memzero(target, len - right..len);
                     }
-                    memzero(target, 0 .. left);
+                    memzero(target, 0..left);
                 }
                 PaddingOption::Center => {
                     let step = if is_complex { 2 } else { 1 };
@@ -472,19 +440,20 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                         left *= 2;
                     }
 
-                    &mut target[len - right .. len].copy_from_slice(&data[len_before - right .. len_before]);
-                    &mut target[0 .. left].copy_from_slice(&data[0 .. left]);
-                    memzero(target, left .. len - len_before);
+                    &mut target[len - right..len]
+                        .copy_from_slice(&data[len_before - right..len_before]);
+                    &mut target[0..left].copy_from_slice(&data[0..left]);
+                    memzero(target, left..len - len_before);
                 }
             }
         }
 
-        mem::swap(&mut self.data, &mut target);
-        buffer.free(target);
+        target.trade(&mut self.data);
+        Ok(())
     }
 
     fn zero_interleave_b<B>(&mut self, buffer: &mut B, factor: u32)
-        where B: Buffer<S, T>
+        where B: for<'a> Buffer<'a, S, T>
     {
         if self.is_complex() {
             zero_interleave!(self, buffer, factor, 2)
@@ -495,7 +464,7 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
 }
 
 impl<S, T, N, D> SplitOps for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
+    where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: Domain
@@ -532,7 +501,7 @@ impl<S, T, N, D> SplitOps for DspVec<S, T, N, D>
 }
 
 impl<S, T, N, D> MergeOps for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
+    where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: Domain
@@ -631,8 +600,8 @@ mod tests {
     fn zero_pad_center_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         v.zero_pad(10, PaddingOption::Center).unwrap();
-        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-            0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+                        0.0, 7.0, 8.0, 9.0, 10.0];
         assert_eq!(&v[..], &expected);
     }
 
@@ -641,15 +610,16 @@ mod tests {
     fn zero_pad_b_center_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
-        v.zero_pad_b(&mut buffer, 10, PaddingOption::Center);
-        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-            0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        v.zero_pad_b(&mut buffer, 10, PaddingOption::Center).unwrap();
+        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+                        0.0, 7.0, 8.0, 9.0, 10.0];
         assert_eq!(&v[..], &expected);
     }
 
     #[test]
     fn zero_pad_surround_odd_signal_test() {
-        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0].to_real_time_vec();
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0]
+            .to_real_time_vec();
         v.zero_pad(20, PaddingOption::Surround).unwrap();
         // The expected result is required so that the convolution theorem holds true
         // (mul in freq is the same as conv)
@@ -662,7 +632,7 @@ mod tests {
     fn zero_pad_b_end_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
-        v.zero_pad_b(&mut buffer, 9, PaddingOption::End);
+        v.zero_pad_b(&mut buffer, 9, PaddingOption::End).unwrap();
         let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 0.0, 0.0, 0.0, 0.0,
                         0.0, 0.0, 0.0, 0.0];
         assert_eq!(&v[..], &expected);
@@ -672,7 +642,7 @@ mod tests {
     fn zero_pad_b_surround_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
-        v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround);
+        v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround).unwrap();
         let expected = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
                         10.0, 0.0, 0.0, 0.0, 0.0];
         assert_eq!(&v[..], &expected);
@@ -680,11 +650,12 @@ mod tests {
 
     #[test]
     fn zero_pad_b_surround_odd_signal_test() {
-        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0, 12.0].to_complex_time_vec();
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0, 12.0]
+            .to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
-        v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround);
-        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
-                        10.0, 11.0, 12.0, 0.0, 0.0, 0.0, 0.0];
+        v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround).unwrap();
+        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0,
+                        11.0, 12.0, 0.0, 0.0, 0.0, 0.0];
         assert_eq!(&v[..], &expected);
     }
 
@@ -712,7 +683,8 @@ mod tests {
     fn zero_pad_surround_overlap_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         v.zero_pad(8, PaddingOption::Surround).unwrap();
-        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 0.0, 0.0];
+        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0,
+                        0.0, 0.0];
         assert_eq!(&v[..], &expected);
     }
 
@@ -720,7 +692,8 @@ mod tests {
     fn zero_pad_center_overlap_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
         v.zero_pad(8, PaddingOption::Center).unwrap();
-        let expected = [ 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.0, 8.0, 9.0,
+                        10.0];
         assert_eq!(&v[..], &expected);
     }
 
