@@ -1,16 +1,16 @@
-use {RealNumber, array_to_complex, Zero};
+use array_to_complex;
 use numbers::*;
-use num_complex::{Complex64};
+use num_complex::Complex64;
 use multicore_support::*;
-use super::super::{Vector, DspVec, ToSlice, Domain, RealNumberSpace,
-                   ComplexNumberSpace, Statistics, Stats};
+use super::super::{Vector, DspVec, ToSlice, Domain, RealNumberSpace, ComplexNumberSpace,
+                   Statistics, Stats, StatsVec, STATS_VEC_CAPACTIY, ScalarResult, ErrorReason};
 use super::{kahan_sum, kahan_sumb};
 
 /// Offers the same functionality as the `StatisticsOps` trait but
 /// the statistics are calculated in a more precise (and slower) way.
-pub trait PreciseStatisticsOps<T>: Sized
-    where T: Sized
-{
+pub trait PreciseStatisticsOps<T> {
+    type Result;
+
     /// Calculates the statistics of the data contained in the vector using
     /// a more precise but slower algorithm.
     ///
@@ -35,13 +35,20 @@ pub trait PreciseStatisticsOps<T>: Sized
     /// assert_eq!(result.max_index, 2);
     /// }
     /// ```
-    fn statistics_prec(&self) -> T;
+    fn statistics_prec(&self) -> Self::Result;
+}
+
+/// Offers the same functionality as the `StatisticsOps` trait but
+/// the statistics are calculated in a more precise (and slower) way.
+pub trait PreciseStatisticsSplitOps<T> {
+    type Result;
 
     /// Calculates the statistics of the data contained in the vector as if the vector would
     /// have been split into `len` pieces
     /// using a more precise but slower algorithm. `self.len` should be dividable by
     /// `len` without a remainder,
     /// but this isn't enforced by the implementation.
+    /// For implementation reasons `len <= 16` must be true.
     ///
     /// # Example
     ///
@@ -53,12 +60,12 @@ pub trait PreciseStatisticsOps<T>: Sized
     /// # fn main() {
     /// let vector: Vec<f32> = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0);
     /// let vector = vector.to_complex_time_vec();
-    /// let result = vector.statistics_splitted_prec(2);
+    /// let result = vector.statistics_split_prec(2).expect("Ignoring error handling in examples");
     /// assert_eq!(result[0].sum, Complex64::new(6.0, 8.0));
     /// assert_eq!(result[1].sum, Complex64::new(3.0, 4.0));
     /// }
     /// ```
-    fn statistics_splitted_prec(&self, len: usize) -> Vec<T>;
+    fn statistics_split_prec(&self, len: usize) -> ScalarResult<Self::Result>;
 }
 
 /// Offers the same functionality as the `SumOps` trait but
@@ -109,11 +116,13 @@ pub trait PreciseStats<T>: Sized {
     fn add_prec(&mut self, elem: T, index: usize, sumc: &mut T, rmsc: &mut T);
 }
 
-impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f32, N, D>
+impl<S, N, D> PreciseStatisticsOps<f64> for DspVec<S, f32, N, D>
     where S: ToSlice<f32>,
           N: RealNumberSpace,
           D: Domain
 {
+    type Result = Statistics<f64>;
+
     fn statistics_prec(&self) -> Statistics<f64> {
         let data_length = self.len();
         let array = self.data.to_slice();
@@ -134,10 +143,22 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f32, N, D>
 
         Statistics::merge(&chunks[..])
     }
+}
 
-    fn statistics_splitted_prec(&self, len: usize) -> Vec<Statistics<f64>> {
+impl<S, N, D> PreciseStatisticsSplitOps<f64> for DspVec<S, f32, N, D>
+    where S: ToSlice<f32>,
+          N: RealNumberSpace,
+          D: Domain
+{
+    type Result = StatsVec<Statistics<f64>>;
+
+    fn statistics_split_prec(&self, len: usize) -> ScalarResult<StatsVec<Statistics<f64>>> {
         if len == 0 {
-            return Vec::new();
+            return Ok(StatsVec::new());
+        }
+
+        if len > STATS_VEC_CAPACTIY {
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
         let data_length = self.len();
@@ -159,15 +180,17 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f32, N, D>
             results
         });
 
-        Statistics::merge_cols(&chunks[..])
+        Ok(Statistics::merge_cols(&chunks[..]))
     }
 }
 
-impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f64, N, D>
+impl<S, N, D> PreciseStatisticsOps<f64> for DspVec<S, f64, N, D>
     where S: ToSlice<f64>,
           N: RealNumberSpace,
           D: Domain
 {
+    type Result = Statistics<f64>;
+
     fn statistics_prec(&self) -> Statistics<f64> {
         let data_length = self.len();
         let array = self.data.to_slice();
@@ -190,10 +213,22 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f64, N, D>
 
         Statistics::merge(&chunks[..])
     }
+}
 
-    fn statistics_splitted_prec(&self, len: usize) -> Vec<Statistics<f64>> {
+impl<S, N, D> PreciseStatisticsSplitOps<f64> for DspVec<S, f64, N, D>
+    where S: ToSlice<f64>,
+          N: RealNumberSpace,
+          D: Domain
+{
+    type Result = StatsVec<Statistics<f64>>;
+
+    fn statistics_split_prec(&self, len: usize) -> ScalarResult<StatsVec<Statistics<f64>>> {
         if len == 0 {
-            return Vec::new();
+            return Ok(StatsVec::new());
+        }
+
+        if len > STATS_VEC_CAPACTIY {
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
         let data_length = self.len();
@@ -217,7 +252,7 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<f64>> for DspVec<S, f64, N, D>
             results
         });
 
-        Statistics::merge_cols(&chunks[..])
+        Ok(Statistics::merge_cols(&chunks[..]))
     }
 }
 
@@ -241,7 +276,8 @@ impl<S, N, D> PreciseSumOps<f64> for DspVec<S, f32, N, D>
             }
             sum
         });
-        (&chunks[..]).iter()
+        (&chunks[..])
+            .iter()
             .fold(0.0, |a, b| a + b)
     }
 
@@ -261,7 +297,8 @@ impl<S, N, D> PreciseSumOps<f64> for DspVec<S, f32, N, D>
             }
             sum
         });
-        (&chunks[..]).iter()
+        (&chunks[..])
+            .iter()
             .fold(0.0, |a, b| a + b)
     }
 }
@@ -279,10 +316,9 @@ impl<S, N, D> PreciseSumOps<f64> for DspVec<S, f64, N, D>
                                                 &array[0..data_length],
                                                 1,
                                                 (),
-                                                move |array, _, _| {
-            kahan_sumb(array.iter())
-        });
-        (&chunks[..]).iter()
+                                                move |array, _, _| kahan_sumb(array.iter()));
+        (&chunks[..])
+            .iter()
             .fold(0.0, |a, b| a + b)
     }
 
@@ -295,18 +331,21 @@ impl<S, N, D> PreciseSumOps<f64> for DspVec<S, f64, N, D>
                                                 1,
                                                 (),
                                                 move |array, _, _| {
-            kahan_sum(array.iter().map(|x|x*x))
-        });
-        (&chunks[..]).iter()
+                                                    kahan_sum(array.iter().map(|x| x * x))
+                                                });
+        (&chunks[..])
+            .iter()
             .fold(0.0, |a, b| a + b)
     }
 }
 
-impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f32, N, D>
+impl<S, N, D> PreciseStatisticsOps<Complex<f64>> for DspVec<S, f32, N, D>
     where S: ToSlice<f32>,
           N: ComplexNumberSpace,
           D: Domain
 {
+    type Result = Statistics<Complex<f64>>;
+
     fn statistics_prec(&self) -> Statistics<Complex<f64>> {
         let data_length = self.len();
         let array = self.data.to_slice();
@@ -328,10 +367,24 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f32,
 
         Statistics::merge(&chunks[..])
     }
+}
 
-    fn statistics_splitted_prec(&self, len: usize) -> Vec<Statistics<Complex<f64>>> {
+impl<S, N, D> PreciseStatisticsSplitOps<Complex<f64>> for DspVec<S, f32, N, D>
+    where S: ToSlice<f32>,
+          N: ComplexNumberSpace,
+          D: Domain
+{
+    type Result = StatsVec<Statistics<Complex<f64>>>;
+
+    fn statistics_split_prec(&self,
+                             len: usize)
+                             -> ScalarResult<StatsVec<Statistics<Complex<f64>>>> {
         if len == 0 {
-            return Vec::new();
+            return Ok(StatsVec::new());
+        }
+
+        if len > STATS_VEC_CAPACTIY {
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
         let data_length = self.len();
@@ -354,15 +407,17 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f32,
             results
         });
 
-        Statistics::merge_cols(&chunks[..])
+        Ok(Statistics::merge_cols(&chunks[..]))
     }
 }
 
-impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f64, N, D>
+impl<S, N, D> PreciseStatisticsOps<Complex<f64>> for DspVec<S, f64, N, D>
     where S: ToSlice<f64>,
           N: ComplexNumberSpace,
           D: Domain
 {
+    type Result = Statistics<Complex<f64>>;
+
     fn statistics_prec(&self) -> Statistics<Complex<f64>> {
         let data_length = self.len();
         let array = self.data.to_slice();
@@ -386,10 +441,24 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f64,
 
         Statistics::merge(&chunks[..])
     }
+}
 
-    fn statistics_splitted_prec(&self, len: usize) -> Vec<Statistics<Complex<f64>>> {
+impl<S, N, D> PreciseStatisticsSplitOps<Complex<f64>> for DspVec<S, f64, N, D>
+    where S: ToSlice<f64>,
+          N: ComplexNumberSpace,
+          D: Domain
+{
+    type Result = StatsVec<Statistics<Complex<f64>>>;
+
+    fn statistics_split_prec(&self,
+                             len: usize)
+                             -> ScalarResult<StatsVec<Statistics<Complex<f64>>>> {
         if len == 0 {
-            return Vec::new();
+            return Ok(StatsVec::new());
+        }
+
+        if len > STATS_VEC_CAPACTIY {
+            return Err(ErrorReason::InvalidArgumentLength);
         }
 
         let data_length = self.len();
@@ -414,7 +483,7 @@ impl<S, N, D> PreciseStatisticsOps<Statistics<Complex<f64>>> for DspVec<S, f64,
             results
         });
 
-        Statistics::merge_cols(&chunks[..])
+        Ok(Statistics::merge_cols(&chunks[..]))
     }
 }
 
@@ -439,7 +508,8 @@ impl<S, N, D> PreciseSumOps<Complex<f64>> for DspVec<S, f32, N, D>
             }
             sum
         });
-        (&chunks[..]).iter()
+        (&chunks[..])
+            .iter()
             .fold(Complex64::zero(), |a, b| a + b)
     }
 
@@ -460,7 +530,8 @@ impl<S, N, D> PreciseSumOps<Complex<f64>> for DspVec<S, f32, N, D>
             }
             sum
         });
-        (&chunks[..]).iter()
+        (&chunks[..])
+            .iter()
             .fold(Complex64::zero(), |a, b| a + b)
     }
 }
@@ -479,10 +550,12 @@ impl<S, N, D> PreciseSumOps<Complex<f64>> for DspVec<S, f64, N, D>
                                                 2,
                                                 (),
                                                 move |array, _, _| {
-            let array = array_to_complex(&array[0..array.len()]);
-            kahan_sumb(array.iter())
-        });
-        (&chunks[..]).iter()
+                                                    let array =
+                                                        array_to_complex(&array[0..array.len()]);
+                                                    kahan_sumb(array.iter())
+                                                });
+        (&chunks[..])
+            .iter()
             .fold(Complex64::zero(), |a, b| a + b)
     }
 
@@ -495,10 +568,12 @@ impl<S, N, D> PreciseSumOps<Complex<f64>> for DspVec<S, f64, N, D>
                                                 2,
                                                 (),
                                                 move |array, _, _| {
-            let array = array_to_complex(&array[0..array.len()]);
-            kahan_sum(array.iter().map(|x|x*x))
-        });
-        (&chunks[..]).iter()
+                                                    let array =
+                                                        array_to_complex(&array[0..array.len()]);
+                                                    kahan_sum(array.iter().map(|x| x * x))
+                                                });
+        (&chunks[..])
+            .iter()
             .fold(Complex64::zero(), |a, b| a + b)
     }
 }
@@ -535,7 +610,11 @@ impl<T> PreciseStats<Complex<T>> for Statistics<Complex<T>>
     where T: RealNumber
 {
     #[inline]
-    fn add_prec(&mut self, elem: Complex<T>, index: usize, sumc: &mut Complex<T>, rmsc: &mut Complex<T>) {
+    fn add_prec(&mut self,
+                elem: Complex<T>,
+                index: usize,
+                sumc: &mut Complex<T>,
+                rmsc: &mut Complex<T>) {
         let y = elem - *sumc;
         let t = self.sum + y;
         *sumc = (t - self.sum) - y;
