@@ -1,6 +1,6 @@
-use RealNumber;
-use super::super::{Owner, ToFreqResult, TransRes, TimeDomain, Buffer, Vector, DataDomain,
-                   MetaData, ResizeOps, DspVec, ToSliceMut, RealNumberSpace, NumberSpace,
+use numbers::*;
+use super::super::{ToFreqResult, TransRes, TimeDomain, Buffer, Vector, DataDomain, MetaData,
+                   ResizeOps, DspVec, ToSliceMut, RealNumberSpace, NumberSpace,
                    RededicateForceOps, ErrorReason, InsertZerosOpsBuffered,
                    FrequencyDomainOperations, TimeDomainOperations};
 use super::fft;
@@ -33,7 +33,7 @@ pub trait TimeToFrequencyDomainOperations<S, T>: ToFreqResult
     ///        assert!(f32::abs(actual[i] - expected[i]) < 1e-4);
     /// }
     /// ```
-    fn plain_fft<B>(self, buffer: &mut B) -> Self::FreqResult where B: Buffer<S, T>;
+    fn plain_fft<B>(self, buffer: &mut B) -> Self::FreqResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs a Fast Fourier Transformation transforming a time domain vector
     /// into a frequency domain vector.
@@ -54,12 +54,12 @@ pub trait TimeToFrequencyDomainOperations<S, T>: ToFreqResult
     ///        assert!(f32::abs(actual[i] - expected[i]) < 1e-4);
     /// }
     /// ```
-    fn fft<B>(self, buffer: &mut B) -> Self::FreqResult where B: Buffer<S, T>;
+    fn fft<B>(self, buffer: &mut B) -> Self::FreqResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Applies a FFT window and performs a Fast Fourier Transformation transforming a time
     /// domain vector into a frequency domain vector.
     fn windowed_fft<B>(self, buffer: &mut B, window: &WindowFunction<T>) -> Self::FreqResult
-        where B: Buffer<S, T>;
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 /// Defines all operations which are valid on `DataVecs` containing real time domain data.
@@ -84,7 +84,8 @@ pub trait SymmetricTimeToFrequencyDomainOperations<S, T>: ToFreqResult
     ///
     /// # Unstable
     /// Symmetric IFFTs are unstable and may only work under certain conditions.
-    fn plain_sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult> where B: Buffer<S, T>;
+    fn plain_sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult>
+        where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs a Symmetric Fast Fourier Transformation under the assumption that `self`
     /// is symmetric around the center. This assumption
@@ -97,7 +98,7 @@ pub trait SymmetricTimeToFrequencyDomainOperations<S, T>: ToFreqResult
     ///
     /// # Unstable
     /// Symmetric IFFTs are unstable and may only work under certain conditions.
-    fn sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult> where B: Buffer<S, T>;
+    fn sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult> where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs a Symmetric Fast Fourier Transformation under the assumption that `self`
     /// is symmetric around the center. This assumption
@@ -114,7 +115,7 @@ pub trait SymmetricTimeToFrequencyDomainOperations<S, T>: ToFreqResult
                         buffer: &mut B,
                         window: &WindowFunction<T>)
                         -> TransRes<Self::FreqResult>
-        where B: Buffer<S, T>;
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 
@@ -123,12 +124,12 @@ impl<S, T, N, D> TimeToFrequencyDomainOperations<S, T> for DspVec<S, T, N, D>
           <DspVec<S, T, N, D> as ToFreqResult>::FreqResult:
             RededicateForceOps<DspVec<S, T, N, D>>
             + FrequencyDomainOperations<S, T>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: TimeDomain {
     fn plain_fft<B>(mut self, buffer: &mut B) -> Self::FreqResult
-        where B: Buffer<S, T> {
+        where B: for<'a> Buffer<'a, S, T> {
         if self.domain() != DataDomain::Time {
             self.valid_len = 0;
             self.number_space.to_complex();
@@ -148,24 +149,24 @@ impl<S, T, N, D> TimeToFrequencyDomainOperations<S, T> for DspVec<S, T, N, D>
     }
 
     fn fft<B>(self, buffer: &mut B) -> Self::FreqResult
-        where B: Buffer<S, T> {
+        where B: for<'a> Buffer<'a, S, T> {
         let mut result = self.plain_fft(buffer);
-        result.fft_shift(buffer);
+        result.fft_shift();
         result
     }
 
     fn windowed_fft<B>(mut self, buffer: &mut B, window: &WindowFunction<T>) -> Self::FreqResult
-        where B: Buffer<S, T> {
+        where B: for<'a> Buffer<'a, S, T> {
         self.apply_window(window);
         let mut result = self.plain_fft(buffer);
-        result.fft_shift(buffer);
+        result.fft_shift();
         result
     }
 }
 
 macro_rules! unmirror {
-    ($self_: ident) => {
-        let len = $self_.points();
+    ($self_: ident, $points: ident) => {
+        let len = $points;
         let len = len / 2 + 1;
         $self_.resize(len).expect("Shrinking a vector should always succeed");
     }
@@ -174,13 +175,13 @@ macro_rules! unmirror {
 impl<S, T, N, D> SymmetricTimeToFrequencyDomainOperations<S, T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToFreqResult,
           <DspVec<S, T, N, D> as ToFreqResult>::FreqResult: RededicateForceOps<DspVec<S, T, N, D>>
-            + FrequencyDomainOperations<S, T> + Vector<T>,
-          S: ToSliceMut<T> + Owner,
+            + FrequencyDomainOperations<S, T> + ResizeOps,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: RealNumberSpace,
           D: TimeDomain {
     fn plain_sfft<B>(mut self, buffer: &mut B) -> TransRes<Self::FreqResult>
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
       if self.domain() != DataDomain::Time ||
          self.is_complex() {
           self.valid_len = 0;
@@ -200,13 +201,14 @@ impl<S, T, N, D> SymmetricTimeToFrequencyDomainOperations<S, T> for DspVec<S, T,
 
       self.zero_interleave_b(buffer, 2);
       self.number_space.to_complex();
+      let points = self.points();
       let mut result = self.plain_fft(buffer);
-      unmirror!(result);
+      unmirror!(result, points);
       Ok(result)
     }
 
     fn sfft<B>(mut self, buffer: &mut B) -> TransRes<Self::FreqResult>
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
       if self.domain() != DataDomain::Time ||
          self.is_complex() {
           self.valid_len = 0;
@@ -226,14 +228,15 @@ impl<S, T, N, D> SymmetricTimeToFrequencyDomainOperations<S, T> for DspVec<S, T,
 
       self.zero_interleave_b(buffer, 2);
       self.number_space.to_complex();
+      let points = self.points();
       let mut result = self.fft(buffer);
-      unmirror!(result);
+      unmirror!(result, points);
       Ok(result)
     }
 
     fn windowed_sfft<B>(mut self, buffer: &mut B, window: &WindowFunction<T>)
         -> TransRes<Self::FreqResult>
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
       if self.domain() != DataDomain::Time ||
          self.is_complex() {
           self.valid_len = 0;
@@ -254,8 +257,9 @@ impl<S, T, N, D> SymmetricTimeToFrequencyDomainOperations<S, T> for DspVec<S, T,
       self.zero_interleave_b(buffer, 2);
       self.number_space.to_complex();
       self.apply_window(window);
+      let points = self.points();
       let mut result = self.fft(buffer);
-      unmirror!(result);
+      unmirror!(result, points);
       Ok(result)
     }
 }
